@@ -2,7 +2,7 @@
 
 
 package EBox::Migration;
-use base 'EBox::MigrationBase';
+use base 'EBox::Migration::Base';
 
 use strict;
 use warnings;
@@ -12,7 +12,7 @@ use EBox::Global;
 use EBox::Gettext;
 use Error qw(:try);
 
-# XXX remove when this block is included in EBox::MigrationBase
+# XXX remove when this block is included in EBox::Migration::Base
 
 # Method: addInternalService
 #
@@ -99,6 +99,7 @@ sub runGConf
 
   $self->addInternalService(
 			    'name' => 'ldap',
+			    'description' => 'LDAP',
 			    'protocol' => 'tcp',
 			    'sourcePort' => 'any',
 			    'destinationPort' => 389,
