@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 
 package EBox::Migration;
-use base 'EBox::MigrationBase';
+use base 'EBox::Migration::Base';
 
 use strict;
 use warnings;
