@@ -22,7 +22,8 @@ use EBox::Exceptions::DataExists;
 use EBox::Exceptions::Internal;
 use EBox::Exceptions::DataNotFound;
 use EBox::Exceptions::Internal;
-use EBox::UsersAndGroups::ImportFromLdif::Engine;
+use EBox::Model::ModelManager;
+
 use EBox::Gettext;
 
 use Net::LDAP;
@@ -38,6 +39,7 @@ use Encode qw( :all );
 use Error qw(:try);
 use File::Slurp qw(read_file write_file);
 use Apache2::RequestUtil;
+use POSIX;
 
 use constant LDAPI         => "ldapi://%2fvar%2frun%2fslapd%2fldapi";
 use constant LDAP          => "ldap://127.0.0.1";
@@ -80,7 +82,7 @@ sub instance
 
 # Method: ldapCon
 #
-#       Returns the Net::LDAP connection
+#       Returns the Net::LDAP connection used by the module
 #
 # Returns:
 #
@@ -108,23 +110,8 @@ sub ldapCon
     }
 
     if ((not defined $self->{ldap}) or $reconnect) {
-        # We try to connect 5 times in 5 seconds, as we might need to
-        # give slapd some time to accept connections after a
-        # slapd restart
-        my $connected = undef;
-        for (0..4) {
-            $self->{ldap} = Net::LDAP->new (LDAPI);
-            if ($self->{ldap}) {
-                $connected = 1;
-                last;
-            } else {
-                sleep (1);
-            }
-        }
-        unless ($connected) {
-            throw EBox::Exceptions::Internal(
-                    "Can't create ldapi connection");
-        }
+        $self->{ldap} = $self->anonymousLdapCon();
+  
         my $global = EBox::Global->getInstance();
         my ($dn, $pass);
         my $auth_type = undef;
@@ -153,6 +140,47 @@ sub ldapCon
     return $self->{ldap};
 }
 
+
+# Method: anonymousLdapCon
+#
+#       returns a LDAP connection without any binding
+#
+# Returns:
+#
+#       An object of class Net::LDAP 
+#
+# Exceptions:
+#
+#       Internal - If connection can't be created
+sub anonymousLdapCon
+{
+    my ($self) = @_;
+    # We try to connect 5 times in 5 seconds, as we might need to
+    # give slapd some time to accept connections after a
+    # slapd restart
+
+    my $ldap;
+
+    my $connected = undef;
+    for (0..4) {
+        $ldap = Net::LDAP->new (LDAPI);
+        if ($ldap) {
+            $connected = 1;
+            last;
+        } else {
+            sleep (1);
+        }
+    }
+
+    unless ($connected) {
+        throw EBox::Exceptions::Internal("Can't create ldapi connection");
+    }
+
+
+    return $ldap;
+}
+
+
 # Method: getPassword
 #
 #       Returns the password used to connect to the LDAP directory
@@ -202,23 +230,26 @@ sub getSlavePassword
 
 # Method: dn
 #
-#       Returns the base dn
+#       Returns the base DN (Distinguished Name)
 #
 # Returns:
 #
-#       string - dn
+#       string - DN
 #
-sub dn {
+sub dn
+{
     my ($self) = @_;
     if(!defined($self->{dn})) {
-        $self->ldapCon();
+        my $ldap = $self->anonymousLdapCon();
+        $ldap->bind();
+
         my %args = (
             'base' => '',
             'scope' => 'base',
             'filter' => '(objectclass=*)',
             'attrs' => ['namingContexts']
         );
-        my $result = $self->{ldap}->search(%args);
+        my $result = $ldap->search(%args);
         my $entry = ($result->entries)[0];
         my $attr = ($entry->attributes)[0];
         $self->{dn} = $entry->get_value($attr);
@@ -266,8 +297,7 @@ sub rootPw
 #
 sub slapdConfFile
 {
-    return SLAPDCONFFILE;
-}
+    return SLAPDCONFFILE;}
 
 # Method: ldapConf
 #
@@ -623,6 +653,52 @@ sub objectClasses
     return [ $result->pop_entry()->get_value('objectClass') ];
 }
 
+# Method: lastModificationTime
+#
+#     Get the last modification time for the directory
+#
+# Parameters:
+#
+#     fromTimestamp - String from timestamp to start the query from to
+#                     speed up the query. If the value is greater than
+#                     the LDAP last modification time, then it returns zero
+#                     *Optional* Default value: undef
+#
+# Returns:
+#
+#     Int - the timestamp in seconds since epoch
+#
+# Example:
+#
+#     $ldap->lastModificationTime('20091204132422Z') => 1259955547
+#
+sub lastModificationTime
+{
+    my ($self, $fromTimestamp) = @_;
+
+    my $filter = '(objectclass=*)';
+    if (defined($fromTimestamp)) {
+        $filter = "(&(objectclass=*)(modifyTimestamp>=$fromTimestamp))";
+    }
+
+    my $res = $self->search({base => $self->dn(), attrs => [ 'modifyTimestamp' ],
+                             filter => $filter });
+    # Order alphanumerically and the latest is the one whose timestamp
+    # is the last one
+    my @sortedEntries = $res->sorted('modifyTimestamp');
+    if ( scalar(@sortedEntries) == 0) {
+        # fromTimestamp given is greater than the current time, so we return 0
+        return 0;
+    }
+    my $lastStamp = $sortedEntries[-1]->get_value('modifyTimestamp');
+
+    # Convert to seconds since epoch
+    my ($year, $month, $day, $h, $m, $s) = 
+      $lastStamp =~ /([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})Z/;
+    return POSIX::mktime( $s, $m, $h, $day, $month -1, $year - 1900 );
+
+}
+
 sub _errorOnLdap
 {
     my ($result, $args) = @_;
@@ -791,39 +867,18 @@ sub _loadLdap
     my $ldifFile = $self->ldifFile($dir, $slapd, $type);
 
     my $backupCommand = $self->_backupSystemDirectory();
+    my $mkCommand = $self->_mkLdapDirCmd($ldapDir);
     my $rmCommand = $self->_rmLdapDirCmd($ldapDir);
     my $slapaddCommand = $self->_slapaddCmd($ldifFile, $slapd, $type);
     my $chownConfCommand = $self->_chownConfDir($slapd);
     my $chownDataCommand = $self->_chownDataDir($slapd);
 
     $self->_execute(
-                cmds => [$backupCommand, $rmCommand,
+                cmds => [$backupCommand, $mkCommand, $rmCommand,
                          $slapaddCommand, $chownConfCommand, $chownDataCommand
                         ]);
 }
 
-# Method: importLdap
-#
-#    import in eBox the data found in LDIF file. Import classes for the various
-#    modules are used to load the data
-#
-#    Parameters:
-#       dir - directory in which is the LDIF file
-sub _importLdap
-{
-    my ($self, $dir, $slapd, $base) = @_;
-
-    my $ldifFile = $self->ldifFile($dir, $slapd, $base);
-
-    EBox::UsersAndGroups::ImportFromLdif::Engine::importLdif($ldifFile);
-}
-
-sub _importLdapData
-{
-    my ($self, $dir, $slapd) = @_;
-    $self->_importLdap($dir, $slapd, $self->dn());
-}
-
 sub _loadLdapData
 {
     my ($self, $dir, $slapd) = @_;
@@ -850,13 +905,40 @@ sub _loadLdapConfig
 sub restoreLdapMaster
 {
     my ($self, $dir) = @_;
-    $self->_loadLdapConfig($dir, 'master');
-    my $ldifFile = $self->ldifFile($dir, 'master', 'data');
-    my $content = read_file($ldifFile);
-    my $passline = 'userPassword: ' . $self->getPassword();
-    $content =~ s/(^dn: cn=ebox,.*?)userPassword:.*?$/$1$passline/ms;
-    write_file($ldifFile, $content);
-    $self->_loadLdapData($dir, 'master');
+
+    if (-f "$dir/ldap.ldif") {
+        EBox::Sudo::root("rm -rf /var/lib/ldap/*");
+
+        my $model = EBox::Model::ModelManager->instance()->model('Mode');
+        my $dn = $model->dnValue();
+        my @parts = split(/,/, $dn);
+        my $dc = (split(/=/, $parts[0]))[1];
+
+        EBox::Sudo::root("cp $dir/ldap.ldif /var/lib/ebox/tmp/ldap.ldif");
+        EBox::Sudo::root("sed -i -e 's/dc=ebox/$dn/' /var/lib/ebox/tmp/ldap.ldif");
+        EBox::Sudo::root("sed -i -e 's/dc: ebox/dc: $dc/' /var/lib/ebox/tmp/ldap.ldif");
+        EBox::Sudo::rootWithoutException("/usr/sbin/slapadd -c -F /etc/ldap/slapd.d < /var/lib/ebox/tmp/ldap.ldif");
+        EBox::Module::Base::writeConfFileNoCheck(EBox::Config::tmp() .
+            'slapd-master-upgrade-ebox.ldif',
+            'usersandgroups/slapd-master-upgrade-ebox.ldif.mas',
+            [
+                'dn' => $dn,
+                'password' => $self->getPassword()
+            ]);
+        EBox::Sudo::root('chown -R '  . LDAP_USER . ':' . LDAP_GROUP . ' /var/lib/ldap');
+        EBox::Sudo::root('/etc/init.d/slapd start');
+        sleep(1);
+        EBox::Sudo::root("ldapadd -H 'ldapi://' -Y EXTERNAL -c -f " .
+            EBox::Config::tmp() . "slapd-master-upgrade-ebox.ldif");
+    } else {
+        $self->_loadLdapConfig($dir, 'master');
+        my $ldifFile = $self->ldifFile($dir, 'master', 'data');
+        my $content = read_file($ldifFile);
+        my $passline = 'userPassword: ' . $self->getPassword();
+        $content =~ s/(^dn: cn=ebox,.*?)userPassword:.*?$/$1$passline/ms;
+        write_file($ldifFile, $content);
+        $self->_loadLdapData($dir, 'master');
+    }
 }
 
 sub restoreLdapReplica
@@ -926,12 +1008,23 @@ sub _slapaddCmd
     return  "/usr/sbin/slapadd -c $options -F " . confDir($slapd) .  " -b '$base' < $ldifFile";
 }
 
+sub _mkLdapDirCmd
+{
+    my ($self, $ldapDir)   = @_;
+
+    return "mkdir -p $ldapDir";
+}
+
 sub _rmLdapDirCmd
 {
     my ($self, $ldapDir)   = @_;
-    $ldapDir .= '/*' if defined $ldapDir ;
 
-    return "sh -c '/bin/rm -rf $ldapDir'";
+    if (defined($ldapDir)) {
+        $ldapDir .= '/*';
+        return "sh -c '/bin/rm -rf $ldapDir'";
+    } else {
+        return "true";
+    }
 }
 
 sub _backupSystemDirectory
