@@ -59,8 +59,8 @@ use constant SYSMINGID      => 1900;
 use constant MINUID         => 2000;
 use constant MINGID         => 2000;
 use constant HOMEPATH       => '/nonexistent';
-use constant MAXUSERLENGTH  => 24;
-use constant MAXGROUPLENGTH => 24;
+use constant MAXUSERLENGTH  => 32;
+use constant MAXGROUPLENGTH => 32;
 use constant MAXPWDLENGTH   => 512;
 use constant SECRETFILE     => '/etc/ldap.secret';
 use constant LDAPCONFDIR    => '/etc/ldap/';
@@ -74,7 +74,7 @@ sub _create
 {
     my $class = shift;
     my $self = $class->SUPER::_create(name => 'users',
-                                      printableName => __('Users and Groups'),
+                                      printableName => __n('Users and Groups'),
                                       domain => 'ebox-usersandgroups',
                                       @_);
 
@@ -88,30 +88,46 @@ sub _create
 #
 sub actions
 {
-    if (isMaster()) {
+    my $mode = mode();
+
+    if ($mode eq 'master') {
         return [
                 {
                  'action' => __('Your openLDAP database will be populated with some basic organizational units'),
                  'reason' => __('eBox needs this organizational units to add users and groups into them'),
-                'module' => 'users'
+                 'module' => 'users'
+            },
+           {
+            'action' => __(q{Create directories for slave's journals}),
+            'reason' => __(q{eBox needs the directories to record pending slave's actions}),
+            'module' => 'users'
             }
         ];
-    } else {
+    } elsif ($mode eq 'slave') {
         my @actions = (
                 {
                  'action' => __('Your eBox will be registered as a slave in the eBox master specified'),
                  'reason' => __('This eBox needs to have remote access to the users in the eBox master'),
-                'module' => 'users'
+                 'module' => 'users'
             }
         );
         if ( -f '/etc/init.d/apparmor' ) {
             push (@actions, {
                'action' => __('Apparmor profile will be disabled'),
-                'reason' => __('It is not ready to work with more than one slapd'),
-                'module' => 'users'
+               'reason' => __('It is not ready to work with more than one slapd'),
+               'module' => 'users'
             });
         }
         return \@actions;
+    } elsif ($mode eq 'ad-slave') {
+        return [
+                {
+                 'action' => __('Install /etc/cron.d/ebox-ad-sync.'),
+                 'reason' => __('eBox will run a script each 5 minutes to sync with Windows AD.'),
+                 'module' => 'users'
+                }
+        ];
+
     }
 }
 
@@ -121,7 +137,9 @@ sub actions
 #
 sub usedFiles
 {
-    if (isMaster()) {
+    my $mode = mode();
+
+    if ($mode ne 'slave') {
         return [
                 {
                  'file' => '/etc/default/slapd',
@@ -148,24 +166,21 @@ sub enableActions
 {
     my ($self) = @_;
 
-    if (isMaster()) {
-        my $password = remotePassword();
+    my $mode = mode();
 
-        EBox::UsersAndGroups::Setup::master($password);
-
-        $self->performLDAPActions();
-    } else {
-        if ( -f '/etc/init.d/apparmor' ) {
-            my $cmd = 'ln -s /etc/apparmor.d/usr.sbin.slapd ' .
-                      '/etc/apparmor.d/disabled/usr.sbin.slapd"';
-            EBox::Sudo::root($cmd);
-            EBox:Sudo::root('invoke-rc.d apparmor restart');
-        }
+    if ($mode eq 'slave') {
+        $self->disableApparmorProfile('usr.sbin.slapd');
 
         EBox::Sudo::root("invoke-rc.d slapd stop");
         EBox::Sudo::root("cp " . EBox::Config::share() . "/ebox-usersandgroups/slapd.default.no /etc/default/slapd");
 
         $self->_setupSlaveLDAP();
+    } else { # Same setup for 'master' and 'ad-slave' modes
+        my $password = remotePassword();
+
+        EBox::UsersAndGroups::Setup::master($password);
+
+        $self->performLDAPActions();
     }
 }
 
@@ -177,12 +192,15 @@ sub _setConf
 {
     my ($self) = @_;
 
+    my $mode = mode();
+
     my $ldap = $self->ldap;
     EBox::Module::Base::writeFile(SECRETFILE, $ldap->getPassword(),
         { mode => '0600', uid => 0, gid => 0 });
 
     my $soapfile = EBox::Config::conf() . "/apache-soap-slave";
-    if( ( not isMaster() ) and ( not -f $soapfile ) ) {
+    if( ( $mode eq 'slave' ) and ( not -f $soapfile ) ) {
+        $self->disableApparmorProfile('usr.sbin.slapd');
         my $apache = EBox::Global->modInstance('apache');
         EBox::Module::Base::writeConfFileNoCheck($soapfile,
             'usersandgroups/soap-slave.mas',
@@ -191,6 +209,11 @@ sub _setConf
         $apache->addInclude($soapfile);
         $apache->save();
     }
+
+    if ( $mode eq 'ad-slave' ) {
+        my $cronFile = EBox::Config::share() . '/ebox-usersandgroups/ebox-ad-sync.cron';
+        EBox::Sudo::root("install -m 0644 -o root -g root $cronFile /etc/cron.d/ebox-ad-sync");
+    }
 }
 
 # Method: _daemons
@@ -199,14 +222,20 @@ sub _setConf
 #
 sub _daemons
 {
-    if (isMaster()) {
+    my $mode = mode();
+
+    if ($mode eq 'master') {
         return [];
-    } else {
+    } elsif ($mode eq 'slave') {
         return [
             { 'name' => 'ebox.slapd-replica' },
             { 'name' => 'ebox.slapd-translucent' },
             { 'name' => 'ebox.slapd-frontend' },
         ];
+    } elsif ($mode eq 'ad-slave') {
+        return [
+            { 'name' => 'ebox.ad-pwdsync' },
+        ];
     }
 }
 
@@ -218,13 +247,17 @@ sub _enforceServiceState
 {
     my ($self) = @_;
 
-    if (isMaster()) {
+    my $mode = mode();
+
+    if ($mode eq 'master') {
         $self->_loadCertificates();
     } else {
         $self->SUPER::_enforceServiceState();
 
-        my ($ldap, $dn) = $self->_connRemoteLDAP();
-        $self->_getCertificates($ldap, $dn);
+        if ($mode eq 'slave') {
+            my ($ldap, $dn) = $self->_connRemoteLDAP();
+            $self->_getCertificates($ldap, $dn);
+        }
     }
 }
 
@@ -234,9 +267,15 @@ sub _loadCertificates
     my $ca;
     my $cert;
 
+    my $ldapca;
+
     $cert = read_file(SSL_DIR . "ssl.cert");
     $ca = $cert;
 
+    EBox::Sudo::root('chown -R ebox.ebox /etc/ldap/ssl');
+    $ldapca = read_file("/etc/ldap/ssl/ssl.cert");
+    EBox::Sudo::root('chown -R openldap.openldap /etc/ldap/ssl');
+
     my $dn = $self->masterDn();
 
     # delete old certs and add new ones
@@ -247,7 +286,8 @@ sub _loadCertificates
     my %args = ('attr' => [
         'objectClass' => 'masterHost',
         'masterCertificate' => $cert,
-        'masterCACertificate' => $ca
+        'masterCACertificate' => $ca,
+        'masterLDAPCACertificate' => $ldapca
     ]);
     $self->ldap->add($dn, \%args);
 }
@@ -264,8 +304,10 @@ sub modelClasses
         'EBox::UsersAndGroups::Model::Groups',
         'EBox::UsersAndGroups::Model::Password',
         'EBox::UsersAndGroups::Model::Slaves',
+        'EBox::UsersAndGroups::Model::LdapInfo',
         'EBox::UsersAndGroups::Model::PendingSync',
         'EBox::UsersAndGroups::Model::ForceSync',
+        'EBox::UsersAndGroups::Model::ADSyncSettings',
     ];
 }
 
@@ -277,6 +319,7 @@ sub compositeClasses
 {
     return [
         'EBox::UsersAndGroups::Composite::SlaveInfo',
+        'EBox::UsersAndGroups::Composite::UserTemplate',
     ];
 }
 # Method: groupsDn
@@ -524,6 +567,26 @@ sub restoreState
     $self->_enforceServiceState();
 }
 
+# Method: configured
+#
+#    Overrides EBox::Module::Service::configured method. The normal method
+#    calls usedFiles() and actions() and it might cause migrations to not work
+#
+sub configured
+{
+    my ($self) = @_;
+
+    if (-d EBox::Config::conf() . "configured/") {
+        return -f (EBox::Config::conf() . "configured/" . $self->name());
+    }
+
+    unless ($self->st_get_bool('_serviceConfigured')) {
+        return undef;
+    }
+
+    return $self->st_get_bool('_serviceConfigured');
+}
+
 sub rewriteObjectClasses
 {
     my ($self, $dn) = @_;
@@ -581,11 +644,14 @@ sub initUser
 
 sub soapClient
 {
-    my ($self, $hostname) = @_;
+    my ($self, $slave) = @_;
+
+    my $hostname = $slave->{'hostname'};
+    my $port = $slave->{'port'};
 
     my $client = EBox::SOAPClient->instance(
         name  => 'urn:EBox/Users',
-        proxy => "https://$hostname/slave",
+        proxy => "https://$hostname:$port/slave",
         certs => {
             cert => SSL_DIR . 'ssl.pem',
             private => SSL_DIR . 'ssl.key'
@@ -596,25 +662,33 @@ sub soapClient
 
 sub soapRun
 {
-    my ($self, $slave, $method, $param) = @_;
+    my ($self, $slave, $method, $param, @params) = @_;
 
-    my $journaldir = EBox::Config::conf() . "userjournal/$slave";
+    my $journaldir = $self->_journalsDir . $slave->{'hostname'};
     (-d $journaldir) or EBox::Sudo::command("mkdir -p $journaldir");
 
     my $client = $self->soapClient($slave);
 
     try {
-        $client->$method($param);
+        $client->$method($param, @params);
     } otherwise {
         EBox::debug("Unable to perform operation $method with parameter $param on slave $slave");
         my ($fh, $filename) = tempfile("$method-XXXX", DIR => $journaldir);
         print $fh "$method\n";
         print $fh "$param\n";
+        for my $p (@params) {
+            print $fh "$p\n";
+        }
         $fh->close();
         rename($filename, "$filename.pending");
     };
 }
 
+sub _journalsDir
+{
+    return EBox::Config::conf() . 'userjournal/';
+}
+
 sub _initUserSlaves
 {
     my ($self, $user) = @_;
@@ -881,7 +955,7 @@ sub modifyUser # (\%user)
     my ($self, $user) = @_;
 
     $self->modifyUserLocal($user);
-    $self->_updateUserSlaves($user->{'username'}, $user->{'password'});
+    $self->_updateUserSlaves($user->{'username'});
 }
 
 # Method: modifyUserLocal
@@ -1101,7 +1175,8 @@ sub uidList
 #
 # Returns:
 #
-#       array - holding the users
+#       array - holding the users. Each user is represented by a hash reference
+#       with the same format than the return value of userInfo
 #
 sub users
 {
@@ -1405,17 +1480,44 @@ sub _checkGid
 
 
 
-
-sub _updateGroup
+sub updateGroup
 {
-    my ($self, $group) = @_;
+    my ($self, $group, @params) = @_;
+
+    $self->updateGroupLocal($group, @params);
+    $self->_updateGroupSlaves($group, @params);
+}
+
+sub updateGroupLocal
+{
+    my ($self, $group, @params) = @_;
 
     # Tell modules depending on groups and groups
     # a group  has been updated
     my @mods = @{$self->_modsLdapUserBase()};
 
     foreach my $mod (@mods){
-        $mod->_modifyGroup($group);
+        $mod->_modifyGroup($group, @params);
+    }
+}
+
+sub _updateGroupSlaves
+{
+    my ($self, $group, @params) = @_;
+
+    for my $slave (@{$self->listSlaves()}) {
+        my $client = $self->soapClient($slave);
+        $self->soapRun($slave, 'updateGroup', $group, @params);
+    }
+}
+
+sub _delGroupSlaves
+{
+    my ($self, $group) = @_;
+
+    for my $slave (@{$self->listSlaves()}) {
+        my $client = $self->soapClient($slave);
+        $self->soapRun($slave, 'delGroup', $group);
     }
 }
 
@@ -1430,7 +1532,7 @@ sub _updateGroup
 #
 sub modifyGroup # (\%groupdata))
 {
-    my ($self, $groupdata) = @_;
+    my ($self, $groupdata, @params) = @_;
 
     my $cn = $groupdata->{'groupname'};
     my $dn = "cn=$cn," . $self->groupsDn;
@@ -1478,6 +1580,22 @@ sub delGroup # (group)
         my $dn = "cn=" . $group . "," . $self->groupsDn;
         my $result = $self->ldap->delete($dn);
 
+    $self->_delGroupSlaves($group);
+}
+
+# Method: delGroupSlave
+#
+#       Removes a given group in a slave
+#
+# Parameters:
+#
+#       group - group name to be deleted
+#
+sub delGroupSlave # (group)
+{
+    my ($self, $group) = @_;
+
+    $self->_cleanGroup($group);
 }
 
 # Method: groupInfo
@@ -1506,7 +1624,7 @@ sub groupInfo # (group)
                 base => $self->groupsDn,
                 filter => "(cn=$group)",
                 scope => 'one',
-                attrs => ['cn', 'description']
+                attrs => ['cn', 'gidNumber', 'description']
                );
 
     my $result = $self->ldap->search(\%args);
@@ -1515,6 +1633,7 @@ sub groupInfo # (group)
     # Mandatory data
     my $groupinfo = {
                      groupname => $entry->get_value('cn'),
+                     gid => $entry->get_value('gidNumber'),
                     };
 
 
@@ -1612,7 +1731,7 @@ sub addUserToGroup # (user, group)
     my %attrs = ( add => { memberUid => $user } );
     $self->ldap->modify($dn, \%attrs);
 
-    $self->_updateGroup($group);
+    $self->updateGroup($group, op => 'add', user => $user);
 }
 
 # Method: delUserFromGroup
@@ -1645,7 +1764,7 @@ sub delUserFromGroup # (user, group)
     my %attrs = ( delete => {  memberUid => $user  } );
         $self->ldap->modify($dn, \%attrs);
 
-    $self->_updateGroup($group);
+    $self->updateGroup($group, op => 'del', user => $user);
 }
 
 # Method: groupsOfUser
@@ -1942,7 +2061,7 @@ sub _checkName
 {
     my ($name) = @_;
 
-    if ($name =~ /^[\w\d\s_-]+\.?[\w\d\s_-]+$/) {
+    if ($name =~ /^[a-zA-Z\d\s_-]+\.?[a-zA-Z\d\s_-]+$/) {
         return 1;
     } else {
         return undef;
@@ -1975,6 +2094,21 @@ sub _modsLdapUserBase
     return \@modules;
 }
 
+# Method: defaultUserModels
+#
+#   Returns all the defaultUserModels from modules implementing
+#   <EBox::LdapUserBase>
+sub defaultUserModels
+{
+    my ($self) = @_;
+    my @models;
+    for my $module  (@{$self->_modsLdapUserBase()}) {
+        my $model = $module->defaultUserModel();
+        push (@models, $model) if (defined($model));
+    }
+    return \@models;
+}
+
 # Method: allUserAddOns
 #
 #       Returns all the mason components from those modules implementing
@@ -2100,7 +2234,8 @@ sub allWarnings
 sub isRunning
 {
     my ($self) = @_;
-    if (isMaster()) {
+
+    if (mode() eq 'master') {
         return $self->isEnabled();
     } else {
         return $self->SUPER::isRunning();
@@ -2124,9 +2259,9 @@ sub menu
 {
     my ($self, $root) = @_;
 
-    my $folder = new EBox::Menu::Folder('name' => 'Users',
-                                        'text' => __('Users'),
-                                        'separator' => __('Office'),
+    my $folder = new EBox::Menu::Folder('name' => 'UsersAndGroups',
+                                        'text' => $self->printableName(),
+                                        'separator' => 'Office',
                                         'order' => 510);
 
     if ($self->configured()) {
@@ -2135,30 +2270,36 @@ sub menu
 
         if ($mode eq 'master') {
             $folder->add(new EBox::Menu::Item('url' => 'UsersAndGroups/Users',
-                                              'text' => __('Add User')));
-        }
-        $folder->add(new EBox::Menu::Item(
-            'url' => '/Users/View/Users',
-            'text' => __('Edit User')));
-        if ($mode eq 'master') {
-	    $folder->add(new EBox::Menu::Item(
-		'url' => 'Users/Composite/SlaveInfo',
-		'text' => __('Slave Status')));
-         }
-
-        $root->add($folder);
-
-        $folder = new EBox::Menu::Folder('name' => 'Group',
-                                     'text' => __('Groups'),
-                                     'separator' => __('Office'),
-                                     'order' => 520);
-
-        if ($mode eq 'master') {
+                                              'text' => __('Users')));
             $folder->add(new EBox::Menu::Item('url' => 'UsersAndGroups/Groups',
-                'text' => __('Add Group')));
+                                              'text' => __('Groups')));
+            $folder->add(new EBox::Menu::Item('url' => 'Users/Composite/UserTemplate',
+                                              'text' => __('Default User Template')));
+        } else {
+            $folder->add(new EBox::Menu::Item(
+                        'url' => '/Users/View/Users',
+                        'text' => __('Users')));
+            $folder->add(new EBox::Menu::Item(
+                        'url' => '/Users/View/Groups',
+                        'text' => __('Groups')));
+            $folder->add(new EBox::Menu::Item('url' => 'Users/Composite/UserTemplate',
+                                              'text' => __('Default User Template')));
         }
-        $folder->add(new EBox::Menu::Item('url' => 'Users/View/Groups',
-            'text' => __('Edit Group')));
+        if (($mode eq 'master') or ($mode eq 'ad-slave')) {
+            $folder->add(new EBox::Menu::Item(
+                'url' => '/Users/View/LdapInfo',
+                'text' => __('LDAP Info')));
+        }
+        if ($mode eq 'master') {
+            $folder->add(new EBox::Menu::Item(
+                        'url' => 'Users/Composite/SlaveInfo',
+                        'text' => __('Slave Status')));
+        } elsif ($mode eq 'ad-slave') {
+            $folder->add(new EBox::Menu::Item(
+                        'url' => 'Users/View/ADSyncSettings',
+                        'text' => __('AD Sync Settings')));
+        }
+
         $root->add($folder);
     } else {
         $folder->add(new EBox::Menu::Item('url' => 'Users/View/Mode',
@@ -2189,13 +2330,15 @@ sub dumpConfig
 {
     my ($self, $dir, %options) = @_;
 
-    if (isMaster()) {
+    my $mode = mode();
+
+    if ($mode eq 'master') {
         $self->ldap->dumpLdapMaster($dir);
         if ($options{bug}) {
-            my $file = $self->ldap->ldifFile($dir, 'slapd.d');
+            my $file = $self->ldap->ldifFile($dir, 'master', 'data');
             $self->_removePasswds($file);
         }
-    } else {
+    } elsif ($mode eq 'slave') {
         $self->ldap->dumpLdapReplica($dir);
         $self->ldap->dumpLdapTranslucent($dir);
         $self->ldap->dumpLdapFrontend($dir);
@@ -2207,7 +2350,7 @@ sub restoreConfig
 {
     my ($self, $dir) = @_;
 
-    if (isMaster()) {
+    unless (mode() eq 'slave') {
         EBox::Sudo::root('/etc/init.d/slapd stop');
         $self->ldap->restoreLdapMaster($dir);
         EBox::Sudo::root('/etc/init.d/slapd start');
@@ -2220,6 +2363,7 @@ sub restoreConfig
         $self->ldap->restoreLdapTranslucent($dir);
         $self->ldap->restoreLdapFrontend($dir);
         $self->_manageService('start');
+        $self->waitSync();
         for my $user ($self->users()) {
             $self->initUser($user->{'username'});
         }
@@ -2360,7 +2504,7 @@ sub defaultPasswordHash
 
     my $format = EBox::Config::configkey('default_password_format');
     if (not defined($format)) {
-        $format = 'shashaHa';
+        $format = 'sha1';
     }
     my $hasher = passwordHasher($format);
     my $hash = $hasher->($password);
@@ -2416,10 +2560,14 @@ sub _registerHostname
         throw EBox::Exceptions::External(__x('A host with the name {host} is already registered in this eBox', host => $hostname));
     }
 
+    my $apache = EBox::Global->modInstance('apache');
+    my $port = $apache->port();
+
     %args = (
         attr => [
             'objectClass' => 'slaveHost',
-            'hostname' => $hostname
+            'hostname' => $hostname,
+            'port' => $port,
         ]
     );
     $result = $ldap->add("hostname=$hostname,ou=slaves,$dn", %args);
@@ -2441,6 +2589,7 @@ sub _getCertificates
     my $entry = ($result->entries)[0];
     my $cert = $entry->get_value('masterCertificate');
     my $cacert = $entry->get_value('masterCACertificate');
+    my $ldapcacert = $entry->get_value('masterLDAPCACertificate');
 
     write_file(SSL_DIR . 'master.cert', $cert);
     write_file(CA_DIR . 'masterca.pem', $cacert);
@@ -2454,6 +2603,13 @@ sub _getCertificates
         }
     }
     EBox::Sudo::command('ln -s masterca.pem ' . CA_DIR . '`openssl x509 -hash -noout -in ' . CA_DIR . 'masterca.pem`.0');
+
+    write_file(CA_DIR . 'masterldapca.pem', $ldapcacert);
+    EBox::Sudo::root('ln -sf ' . CA_DIR . 'masterldapca.pem /etc/ldap/ssl/masterldapca.pem');
+    EBox::Module::Base::writeConfFileNoCheck('/etc/ldap/ldap.conf',
+            'usersandgroups/ldap.conf.mas',
+    );
+
 }
 
 sub _setupReplication
@@ -2499,7 +2655,7 @@ sub waitSync
 {
     my ($self) = @_;
 
-    if ($self->isMaster()) {
+    unless (mode() eq 'slave') {
         return;
     }
 
@@ -2708,10 +2864,38 @@ sub listSlaves
     );
     my $result = $self->ldap->search(\%args);
 
-    my @slaves = map { $_->get_value('hostname') } $result->entries();
+    my @slaves = map {
+        {
+            'hostname' => $_->get_value('hostname'),
+            'port' => $_->get_value('port')
+        }
+    } $result->entries();
     return \@slaves;
 }
 
+sub slaveInfo
+{
+    my ($self, $slave) = @_;
+
+    my %args = (
+        'base' => $self->slavesDn(),
+        'scope' => 'sub',
+        'filter' => "(hostname=$slave)"
+    );
+    my $result = $self->ldap->search(\%args);
+
+    my $slaveInfo;
+
+    if ($result->count()) {
+        my $entry = ($result->entries)[0];
+        $slaveInfo = {
+            'hostname' => $entry->get_value('hostname'),
+            'port' => $entry->get_value('port')
+        };
+    }
+    return $slaveInfo;
+}
+
 sub deleteSlave
 {
     my ($self, $slave) = @_;
@@ -2725,12 +2909,12 @@ sub deleteSlave
     return $res;
 }
 
-sub isMaster
+sub mode
 {
     my $model = EBox::Model::ModelManager->instance()->model('Mode');
     my $mode = $model->modeValue();
 
-    return ($mode eq 'master');
+    return $mode;
 }
 
 sub remoteLdap
